@@ -245,7 +245,7 @@ static int state_print(DBusMessageIter *iter, const char *error,
 	DBusMessageIter entry;
 
 	if (error) {
-		fprintf(stderr, "Error: %s", error);
+		fprintf(stderr, "Error: %s\n", error);
 		return 0;
 	}
 
@@ -272,7 +272,7 @@ static int clock_print(DBusMessageIter *iter, const char *error,
 	DBusMessageIter entry;
 
 	if (error) {
-		fprintf(stderr, "Error: %s", error);
+		fprintf(stderr, "Error: %s\n", error);
 		return 0;
 	}
 
@@ -901,6 +901,13 @@ struct config_append {
 	int values;
 };
 
+struct session_options {
+	char **args;
+	int num;
+	char *notify_path;
+	struct connman_option *options;
+};
+
 static void config_append_ipv4(DBusMessageIter *iter,
 		void *user_data)
 {
@@ -1202,6 +1209,30 @@ static int cmd_config(char *args[], int num, struct connman_option *options)
 					config_return, g_strdup(service_name),
 					NULL, NULL);
 			break;
+
+		case 'm':
+			switch (parse_boolean(*opt_start)) {
+			case 1:
+				val = TRUE;
+				break;
+			case 0:
+				val = FALSE;
+				break;
+			default:
+				res = -EINVAL;
+				break;
+			}
+			if (res == 0) {
+				res = __connmanctl_dbus_set_property(connection,
+						path, "net.connman.Service",
+						config_return,
+						g_strdup(service_name),
+						"mDNS.Configuration",
+						DBUS_TYPE_BOOLEAN, &val);
+			}
+			index++;
+			break;
+
 		default:
 			res = -EINVAL;
 			break;
@@ -1735,7 +1766,7 @@ static int session_connect_cb(DBusMessageIter *iter, const char *error,
 		void *user_data)
 {
 	if (error) {
-		fprintf(stderr, "Error: %s", error);
+		fprintf(stderr, "Error: %s\n", error);
 		return 0;
 	}
 
@@ -1754,7 +1785,7 @@ static int session_disconnect_cb(DBusMessageIter *iter, const char *error,
 		void *user_data)
 {
 	if (error)
-		fprintf(stderr, "Error: %s", error);
+		fprintf(stderr, "Error: %s\n", error);
 
 	return 0;
 }
@@ -1796,28 +1827,142 @@ static int session_create_cb(DBusMessageIter *iter, const char *error,
 	return -EINPROGRESS;
 }
 
-static void session_create_append(DBusMessageIter *iter, void *user_data)
+static void session_config_append_array(DBusMessageIter *iter,
+		void *user_data)
 {
-	const char *notify_path = user_data;
+	struct config_append *append = user_data;
+	char **opts = append->opts;
+	int i = 1;
 
-	__connmanctl_dbus_append_dict(iter, NULL, NULL);
+	if (!opts)
+		return;
 
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-			&notify_path);
+	while (opts[i] && strncmp(opts[i], "--", 2) != 0) {
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING,
+					       &opts[i]);
+		i++;
+	}
+
+	append->values = i;
 }
 
-static int session_create(gboolean connect)
+static void session_create_append_dict(DBusMessageIter *iter, void *user_data)
+{
+	struct session_options *args_struct = user_data;
+	int index = 0, res = 0;
+	struct config_append append;
+	char c;
+	char *ifname;
+	dbus_bool_t source_ip_rule;
+
+	while (index < args_struct->num && args_struct->args[index]) {
+		append.opts = &args_struct->args[index];
+		append.values = 0;
+
+		c = parse_args(args_struct->args[index], args_struct->options);
+
+		switch (c) {
+		case 'b':
+			__connmanctl_dbus_append_dict_string_array(iter, "AllowedBearers",
+								   session_config_append_array,
+								   &append);
+			break;
+		case 't':
+			if (! args_struct->args[index + 1]) {
+				res = -EINVAL;
+				break;
+			}
+			__connmanctl_dbus_append_dict_entry(iter, "ConnectionType",
+							    DBUS_TYPE_STRING,
+							    &args_struct->args[index + 1]);
+			append.values = 2;
+			break;
+		case 'i':
+			if (index + 1 <  args_struct->num)
+				ifname =  args_struct->args[index + 1];
+			else
+				ifname = "";
+			 __connmanctl_dbus_append_dict_entry(iter, "AllowedInterface",
+							     DBUS_TYPE_STRING,
+							     &ifname);
+			append.values = 2;
+			break;
+		case 's':
+			if (! args_struct->args[index + 1]) {
+				res = -EINVAL;
+				break;
+			}
+			switch (parse_boolean( args_struct->args[index + 1])) {
+			case 1:
+				source_ip_rule = TRUE;
+				break;
+			case 0:
+				source_ip_rule = FALSE;
+				break;
+			default:
+				res = -EINVAL;
+				break;
+			}
+			__connmanctl_dbus_append_dict_entry(iter, "SourceIPRule",
+							    DBUS_TYPE_BOOLEAN,
+							    &source_ip_rule);
+			append.values = 2;
+			break;
+		case 'c':
+			if (!args_struct->args[index + 1]) {
+				res = -EINVAL;
+				break;
+			}
+			__connmanctl_dbus_append_dict_entry(iter, "ContextIdentifier",
+							    DBUS_TYPE_STRING,
+							    &args_struct->args[index + 1]);
+			append.values = 2;
+			break;
+		default:
+			res = -EINVAL;
+		}
+
+		if (res < 0 && res != -EINPROGRESS) {
+			printf("Error '%s': %s\n",  args_struct->args[index],
+					strerror(-res));
+			return;
+		}
+
+		index += append.values;
+	}
+
+	return;
+}
+
+static void session_create_append(DBusMessageIter *iter, void *user_data)
+{
+	struct session_options *args_struct = user_data;
+
+	__connmanctl_dbus_append_dict(iter, session_create_append_dict,
+				      args_struct);
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
+				       &args_struct->notify_path);
+}
+
+static int session_create(gboolean connect, char *args[], int num,
+			  struct connman_option *options)
 {
 	int res;
 	char *notify_path;
+	struct session_options args_struct;
+	args_struct.args = args;
+	args_struct.num = num;
+	args_struct.options = options;
 
 	notify_path = g_strdup_printf("/net/connman/connmanctl%d", getpid());
 	session_notify_add(notify_path);
+	args_struct.notify_path = notify_path;
 
 	res = __connmanctl_dbus_method_call(connection, "net.connman", "/",
 			"net.connman.Manager", "CreateSession",
 			session_create_cb, GINT_TO_POINTER(connect),
-			session_create_append, notify_path);
+			session_create_append, &args_struct);
 
 	g_free(notify_path);
 
@@ -1871,31 +2016,14 @@ static int session_config_return(DBusMessageIter *iter, const char *error,
 	return 0;
 }
 
-static void session_config_append_array(DBusMessageIter *iter,
-		void *user_data)
-{
-	struct config_append *append = user_data;
-	char **opts = append->opts;
-	int i = 1;
-
-	if (!opts)
-		return;
-
-	while (opts[i] && strncmp(opts[i], "--", 2) != 0) {
-		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING,
-				&opts[i]);
-		i++;
-	}
-
-	append->values = i;
-}
-
 static int session_config(char *args[], int num,
 		struct connman_option *options)
 {
 	int index = 0, res = 0;
 	struct config_append append;
 	char c;
+	char *ifname;
+	dbus_bool_t source_ip_rule;
 
 	while (index < num && args[index]) {
 		append.opts = &args[index];
@@ -1922,6 +2050,53 @@ static int session_config(char *args[], int num,
 					DBUS_TYPE_STRING, &args[index + 1]);
 			append.values = 2;
 			break;
+		case 'i':
+			if (index + 1 < num)
+				ifname = args[index + 1];
+			else
+				ifname = "";
+
+			res = __connmanctl_dbus_session_change(connection,
+					session_path, session_config_return,
+					"AllowedInterface", "AllowedInterface",
+					DBUS_TYPE_STRING, &ifname);
+			append.values = 2;
+			break;
+		case 's':
+			if (!args[index + 1]) {
+				res = -EINVAL;
+				break;
+			}
+			switch (parse_boolean(args[index + 1])) {
+			case 1:
+				source_ip_rule = TRUE;
+				break;
+			case 0:
+				source_ip_rule = FALSE;
+				break;
+			default:
+				res = -EINVAL;
+				break;
+			}
+
+			res = __connmanctl_dbus_session_change(connection,
+					session_path, session_config_return,
+					"SourceIPRule", "SourceIPRule",
+					DBUS_TYPE_BOOLEAN, &source_ip_rule);
+			append.values = 2;
+			break;
+		case 'c':
+				if (!args[index + 1]) {
+					res = -EINVAL;
+					break;
+				}
+
+				res = __connmanctl_dbus_session_change(connection,
+						session_path, session_config_return,
+						"ctxid", "ctxid", DBUS_TYPE_STRING,
+						&args[index + 1]);
+				append.values = 2;
+				break;
 
 		default:
 			res = -EINVAL;
@@ -1957,12 +2132,13 @@ static int cmd_session(char *args[], int num, struct connman_option *options)
 	case 1:
 		if (session_path)
 			return -EALREADY;
-		return session_create(FALSE);
+		return session_create(FALSE, &args[2], num - 2, options);
 
 	default:
 		if (!strcmp(command, "connect")) {
 			if (!session_path)
-				return session_create(TRUE);
+				return session_create(TRUE, &args[2], num - 2,
+						      options);
 
 			return session_connect();
 
@@ -2185,6 +2361,7 @@ static struct connman_option config_options[] = {
 	{"nameservers", 'n', "<dns1> [<dns2>] [<dns3>]"},
 	{"timeservers", 't', "<ntp1> [<ntp2>] [...]"},
 	{"domains", 'd', "<domain1> [<domain2>] [...]"},
+	{"mdns", 'm', "yes|no"},
 	{"ipv6", 'v', "off|auto [enable|disable|preferred]|\n"
 	              "\t\t\tmanual <address> <prefixlength> <gateway>"},
 	{"proxy", 'x', "direct|auto <URL>|manual <URL1> [<URL2>] [...]\n"
@@ -2209,6 +2386,9 @@ static struct connman_option monitor_options[] = {
 static struct connman_option session_options[] = {
 	{"bearers", 'b', "<technology1> [<technology2> [...]]"},
 	{"type", 't', "local|internet|any"},
+	{"ifname", 'i', "[<interface_name>]"},
+	{"srciprule", 's', "yes|no"},
+	{"ctxid", 'c', "<context_identifier>"},
 	{ NULL, }
 };
 
@@ -2973,7 +3153,7 @@ static int populate_technology_hash(DBusMessageIter *iter, const char *error,
 				void *user_data)
 {
 	if (error) {
-		fprintf(stderr, "Error getting technologies: %s", error);
+		fprintf(stderr, "Error getting technologies: %s\n", error);
 		return 0;
 	}
 
