@@ -511,7 +511,7 @@ static void sort_and_return_results(struct resolv_lookup *lookup)
 			status = lookup->ipv4_status;
 	}
 
-	debug(lookup->resolv, "lookup %p received %d results", lookup, n);
+	debug(lookup->resolv, "lookup %p received %d results", lookup, n-1);
 
 	g_queue_remove(lookup->resolv->lookup_queue, lookup);
 	destroy_lookup(lookup);
@@ -679,7 +679,10 @@ static void parse_response(struct resolv_nameserver *nameserver,
 
 	switch (rcode) {
 	case ns_r_noerror:
-		status = G_RESOLV_RESULT_STATUS_SUCCESS;
+		if (count > 0)
+			status = G_RESOLV_RESULT_STATUS_SUCCESS;
+		else
+			status = G_RESOLV_RESULT_STATUS_NO_ANSWER;
 		break;
 	case ns_r_formerr:
 		status = G_RESOLV_RESULT_STATUS_FORMAT_ERROR;
