@@ -82,7 +82,8 @@ int connman_device_set_powered(struct connman_device *device,
 bool connman_device_get_powered(struct connman_device *device);
 int connman_device_set_scanning(struct connman_device *device,
 				enum connman_service_type type, bool scanning);
-bool connman_device_get_scanning(struct connman_device *device);
+bool connman_device_get_scanning(struct connman_device *device,
+				enum connman_service_type type);
 void connman_device_reset_scanning(struct connman_device *device);
 
 int connman_device_set_string(struct connman_device *device,
@@ -124,6 +125,8 @@ struct connman_device_driver {
 			const char *ssid, unsigned int ssid_len,
 			const char *identity, const char* passphrase,
 			const char *security, void *user_data);
+	void (*stop_scan) (enum connman_service_type type,
+			struct connman_device *device);
 	int (*set_regdom) (struct connman_device *device,
 						const char *alpha2);
 };
