@@ -103,11 +103,6 @@ bool connman_network_get_connected(struct connman_network *network);
 
 bool connman_network_get_associating(struct connman_network *network);
 
-bool connman_network_get_connectable(struct connman_network *network);
-
-int connman_network_set_connectable(struct connman_network *network,
-		bool connectable);
-
 void connman_network_clear_hidden(void *user_data);
 int connman_network_connect_hidden(struct connman_network *network,
 			char *identity, char* passphrase, void *user_data);
