@@ -41,6 +41,13 @@
  */
 #define RS_REFRESH_TIMEOUT	3
 
+/*
+ * As per RFC 4861, a host should transmit up to MAX_RTR_SOLICITATIONS(3)
+ * Router Solicitation messages, each separated by at least
+ * RTR_SOLICITATION_INTERVAL(4) seconds to obtain RA for IPv6 auto-configuration.
+ */
+#define RTR_SOLICITATION_INTERVAL	4
+
 static GSList *network_list = NULL;
 static GSList *driver_list = NULL;
 
@@ -90,6 +97,7 @@ struct connman_network {
 		char *private_key_passphrase;
 		char *phase2_auth;
 		bool wps;
+		bool wps_advertizing;
 		bool use_wps;
 		char *pin_wps;
 	} wifi;
@@ -427,7 +435,7 @@ static void check_dhcpv6(struct nd_router_advert *reply,
 			DBG("re-send router solicitation %d",
 						network->router_solicit_count);
 			network->router_solicit_count--;
-			__connman_inet_ipv6_send_rs(network->index, 1,
+			__connman_inet_ipv6_send_rs(network->index, RTR_SOLICITATION_INTERVAL,
 						check_dhcpv6, network);
 			return;
 		}
@@ -577,7 +585,8 @@ static void autoconf_ipv6_set(struct connman_network *network)
 
 	/* Try to get stateless DHCPv6 information, RFC 3736 */
 	network->router_solicit_count = 3;
-	__connman_inet_ipv6_send_rs(index, 1, check_dhcpv6, network);
+	__connman_inet_ipv6_send_rs(index, RTR_SOLICITATION_INTERVAL,
+			check_dhcpv6, network);
 }
 
 static void set_connected(struct connman_network *network)
@@ -1459,10 +1468,10 @@ int __connman_network_connect(struct connman_network *network)
 	if (!network->device)
 		return -ENODEV;
 
-	network->connecting = true;
-
 	__connman_device_disconnect(network->device);
 
+	network->connecting = true;
+
 	err = network->driver->connect(network);
 	if (err < 0) {
 		if (err == -EINPROGRESS)
@@ -1919,6 +1928,8 @@ int connman_network_set_bool(struct connman_network *network,
 		network->roaming = value;
 	else if (g_strcmp0(key, "WiFi.WPS") == 0)
 		network->wifi.wps = value;
+	else if (g_strcmp0(key, "WiFi.WPSAdvertising") == 0)
+		network->wifi.wps_advertizing = value;
 	else if (g_strcmp0(key, "WiFi.UseWPS") == 0)
 		network->wifi.use_wps = value;
 
@@ -1939,6 +1950,8 @@ bool connman_network_get_bool(struct connman_network *network,
 		return network->roaming;
 	else if (g_str_equal(key, "WiFi.WPS"))
 		return network->wifi.wps;
+	else if (g_str_equal(key, "WiFi.WPSAdvertising"))
+		return network->wifi.wps_advertizing;
 	else if (g_str_equal(key, "WiFi.UseWPS"))
 		return network->wifi.use_wps;
 
