@@ -606,6 +606,9 @@ static int peer_connect(struct connman_peer *peer)
 {
 	int err = -ENOTSUP;
 
+	if (is_connected(peer))
+		return -EISCONN;
+
 	if (peer_driver->connect)
 		err = peer_driver->connect(peer,
 					CONNMAN_PEER_WPS_UNKNOWN, NULL);
