@@ -787,6 +787,7 @@ static int technology_disable(struct connman_technology *technology)
 
 	if (technology->type == CONNMAN_SERVICE_TYPE_P2P) {
 		technology->enable_persistent = false;
+		__connman_device_stop_scan(CONNMAN_SERVICE_TYPE_P2P);
 		return technology_disabled(technology);
 	} else if (technology->type == CONNMAN_SERVICE_TYPE_WIFI) {
 		struct connman_technology *p2p;
@@ -1022,10 +1023,7 @@ void __connman_technology_scan_stopped(struct connman_device *device,
 		if (device == other_device)
 			continue;
 
-		if (__connman_device_get_service_type(other_device) != type)
-			continue;
-
-		if (connman_device_get_scanning(other_device))
+		if (connman_device_get_scanning(other_device, type))
 			count += 1;
 	}
 
