@@ -51,6 +51,9 @@ struct vpn_driver {
 	int (*error_code) (struct vpn_provider *provider, int exit_code);
 	int (*save) (struct vpn_provider *provider, GKeyFile *keyfile);
 	int (*device_flags) (struct vpn_provider *provider);
+	int (*route_env_parse) (struct vpn_provider *provider, const char *key,
+			int *family, unsigned long *idx,
+			enum vpn_provider_route_type *type);
 };
 
 int vpn_register(const char *name, struct vpn_driver *driver,
